@@ -52,20 +52,21 @@ PICKLESHARE_REQVER = '>=0.4'
 PSUTIL_REQVER = '>=5.3'
 PYGMENTS_REQVER = '>=2.0'
 PYLINT_REQVER = '>=2.5.0;<3.0'
-PYLSP_REQVER = '>=1.5.0'
+PYLINT_VENV_REQVER = '>=2.1.1'
+PYLSP_REQVER = '>=1.6.0;<1.7.0'
 PYLSP_BLACK_REQVER = '>=1.2.0'
 PYLS_SPYDER_REQVER = '>=0.4.0'
 PYXDG_REQVER = '>=0.26'
 PYZMQ_REQVER = '>=22.1.0'
 QDARKSTYLE_REQVER = '>=3.0.2'
-QSTYLIZER_REQVER = '>=0.1.10'
-QTAWESOME_REQVER = '>=1.0.2'
-QTCONSOLE_REQVER = '>=5.3.0'
+QSTYLIZER_REQVER = '>=0.2.2'
+QTAWESOME_REQVER = '>=1.2.1'
+QTCONSOLE_REQVER = '>=5.4.0;<5.5.0'
 QTPY_REQVER = '>=2.1.0'
 RTREE_REQVER = '>=0.9.7'
 SETUPTOOLS_REQVER = '>=49.6.0'
 SPHINX_REQVER = '>=0.6.6'
-SPYDER_KERNELS_REQVER = '>=2.3.2;<2.4.0'
+SPYDER_KERNELS_REQVER = '>=2.4.0;<2.5.0'
 TEXTDISTANCE_REQVER = '>=4.2.0'
 THREE_MERGE_REQVER = '>=0.1.1'
 # None for pynsist install for now
@@ -178,6 +179,11 @@ DESCRIPTIONS = [
      'package_name': "pylint",
      'features': _("Static code analysis"),
      'required_version': PYLINT_REQVER},
+    {'modname': "pylint_venv",
+     'package_name': "pylint-venv",
+     'features': _("Use the same Pylint installation with different virtual"
+                   " environments"),
+     'required_version': PYLINT_VENV_REQVER},
     {'modname': 'pylsp',
      'package_name': 'python-lsp-server',
      'features': _("Code completion and linting for the Editor"),
