@@ -14,6 +14,7 @@ import functools
 import logging
 import os
 import os.path as osp
+import sys
 
 # Third-party imports
 from qtpy.QtCore import Signal, Slot, QTimer
@@ -147,6 +148,8 @@ class LanguageServerProvider(SpyderCompletionProvider):
             try:
                 if self.clients_hearbeat[language] is not None:
                     self.clients_hearbeat[language].stop()
+                    self.clients_hearbeat[language].setParent(None)
+                    del self.clients_hearbeat[language]
             except (TypeError, KeyError, RuntimeError):
                 pass
 
@@ -183,11 +186,12 @@ class LanguageServerProvider(SpyderCompletionProvider):
                 self.clients_restart_timers[language] = None
                 try:
                     self.clients_hearbeat[language].stop()
+                    self.clients_hearbeat[language].setParent(None)
+                    del self.clients_hearbeat[language]
                     client['instance'].disconnect()
                     client['instance'].stop()
                 except (TypeError, KeyError, RuntimeError):
                     pass
-                self.clients_hearbeat[language] = None
                 self.report_lsp_down(language)
 
     def create_statusbar(self, parent):
@@ -260,6 +264,8 @@ class LanguageServerProvider(SpyderCompletionProvider):
                 and not running_under_pytest()):
             try:
                 self.clients_hearbeat[language].stop()
+                self.clients_hearbeat[language].setParent(None)
+                del self.clients_hearbeat[language]
             except KeyError:
                 pass
             logger.info("Automatic restart for {}...".format(language))
@@ -463,12 +469,15 @@ class LanguageServerProvider(SpyderCompletionProvider):
 
             started = language_client['status'] == self.RUNNING
 
-            # Start client heartbeat
-            timer = QTimer(self)
-            self.clients_hearbeat[language] = timer
-            timer.setInterval(self.TIME_HEARTBEAT)
-            timer.timeout.connect(functools.partial(self.check_heartbeat, language))
-            timer.start()
+            if language not in self.clients_hearbeat:
+                # completion_services for language is already running
+                # Start client heartbeat
+                timer = QTimer(self)
+                self.clients_hearbeat[language] = timer
+                timer.setInterval(self.TIME_HEARTBEAT)
+                timer.timeout.connect(functools.partial(
+                    self.check_heartbeat, language))
+                timer.start()
 
             if language_client['status'] == self.STOPPED:
                 config = language_client['config']
@@ -658,6 +667,8 @@ class LanguageServerProvider(SpyderCompletionProvider):
                 try:
                     if self.clients_hearbeat[language] is not None:
                         self.clients_hearbeat[language].stop()
+                        self.clients_hearbeat[language].setParent(None)
+                        del self.clients_hearbeat[language]
                 except (TypeError, KeyError, RuntimeError):
                     pass
                 language_client['instance'].stop()
@@ -799,15 +810,16 @@ class LanguageServerProvider(SpyderCompletionProvider):
         }
 
         # Jedi configuration
+        env_vars = os.environ.copy()  # Ensure env is indepependent of PyLSP's
+        env_vars.pop('PYTHONPATH', None)
         if self.get_conf('default', section='main_interpreter'):
-            environment = None
-            env_vars = None
+            # If not explicitly set, jedi uses PyLSP's sys.path instead of
+            # sys.executable's sys.path. This may be a bug in jedi.
+            environment = sys.executable
         else:
             environment = self.get_conf('executable',
                                         section='main_interpreter')
-            env_vars = os.environ.copy()
-            # external interpreter should not use internal PYTHONPATH
-            env_vars.pop('PYTHONPATH', None)
+            # External interpreter cannot have PYTHONHOME
             if running_in_mac_app():
                 env_vars.pop('PYTHONHOME', None)
 
