@@ -13,11 +13,8 @@ Client widget for the IPython Console.
 This is the widget used on all its tabs.
 """
 
-
 # Standard library imports.
-# Fix for spyder-ide/spyder#1356.
-from __future__ import absolute_import
-
+import logging
 import os
 import os.path as osp
 import re
@@ -31,7 +28,8 @@ from qtpy.QtWidgets import (QMessageBox, QVBoxLayout, QWidget)
 # Local imports
 from spyder.api.translations import get_translation
 from spyder.api.widgets.mixins import SpyderWidgetMixin
-from spyder.config.base import get_module_source_path, running_under_pytest
+from spyder.config.base import (
+    get_home_dir, get_module_source_path, running_under_pytest)
 from spyder.utils.icon_manager import ima
 from spyder.utils import sourcecode
 from spyder.utils.image_path_manager import get_image_path
@@ -39,15 +37,20 @@ from spyder.utils.installers import InstallerIPythonKernelError
 from spyder.utils.encoding import get_coding
 from spyder.utils.environ import RemoteEnvDialog
 from spyder.utils.palette import QStylePalette
+from spyder.utils import programs
 from spyder.utils.qthelpers import add_actions, DialogManager
 from spyder.py3compat import to_text_string
+from spyder.plugins.ipythonconsole import (
+    SPYDER_KERNELS_CONDA, SPYDER_KERNELS_MAX_VERSION,
+    SPYDER_KERNELS_MIN_VERSION, SPYDER_KERNELS_PIP, SPYDER_KERNELS_VERSION)
 from spyder.plugins.ipythonconsole.widgets import ShellWidget
 from spyder.widgets.collectionseditor import CollectionsEditor
 from spyder.widgets.mixins import SaveHistoryMixin
 
 
-# Localization
+# Localization and logging
 _ = get_translation('spyder')
+logger = logging.getLogger(__name__)
 
 # -----------------------------------------------------------------------------
 # Templates
@@ -63,6 +66,9 @@ TEMPLATES_PATH = osp.join(
 BLANK = open(osp.join(TEMPLATES_PATH, 'blank.html')).read()
 LOADING = open(osp.join(TEMPLATES_PATH, 'loading.html')).read()
 KERNEL_ERROR = open(osp.join(TEMPLATES_PATH, 'kernel_error.html')).read()
+SPYDER_KERNELS_VERSION_MSG = _(
+    '>= {0} and < {1}').format(
+        SPYDER_KERNELS_MIN_VERSION, SPYDER_KERNELS_MAX_VERSION)
 
 try:
     time.monotonic  # time.monotonic new in 3.3
@@ -82,6 +88,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     sig_append_to_history_requested = Signal(str, str)
     sig_execution_state_changed = Signal()
+    sig_time_label = Signal(str)
 
     CONF_SECTION = 'ipython_console'
     SEPARATOR = '{0}## ---({1})---'.format(os.linesep*2, time.ctime())
@@ -99,17 +106,16 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                  given_name=None,
                  give_focus=True,
                  options_button=None,
-                 time_label=None,
                  show_elapsed_time=False,
                  reset_warning=True,
                  ask_before_restart=True,
                  ask_before_closing=False,
                  css_path=None,
-                 configuration=None,
                  handlers={},
                  stderr_obj=None,
                  stdout_obj=None,
-                 fault_obj=None):
+                 fault_obj=None,
+                 initial_cwd=None):
         super(ClientWidget, self).__init__(parent)
         SaveHistoryMixin.__init__(self, history_filename)
 
@@ -125,10 +131,10 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.reset_warning = reset_warning
         self.ask_before_restart = ask_before_restart
         self.ask_before_closing = ask_before_closing
+        self.initial_cwd = initial_cwd
 
         # --- Other attrs
         self.context_menu_actions = context_menu_actions
-        self.time_label = time_label
         self.options_button = options_button
         self.history = []
         self.allow_rename = True
@@ -151,8 +157,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             is_external_kernel=is_external_kernel,
             is_spyder_kernel=is_spyder_kernel,
             handlers=handlers,
-            local_kernel=True,
-            configuration=configuration)
+            local_kernel=True
+        )
         self.infowidget = self.container.infowidget
         self.blank_page = self._create_blank_page()
         self.loading_page = self._create_loading_page()
@@ -169,7 +175,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.layout = QVBoxLayout()
         self.layout.setContentsMargins(0, 0, 0, 0)
         self.layout.addWidget(self.shellwidget)
-        self.layout.addWidget(self.infowidget)
+        if self.infowidget is not None:
+            self.layout.addWidget(self.infowidget)
         self.setLayout(self.layout)
 
         # --- Exit function
@@ -182,12 +189,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.stderr_obj = stderr_obj
         self.stdout_obj = stdout_obj
         self.fault_obj = fault_obj
-        self.std_poll_timer = None
         if self.stderr_obj is not None or self.stdout_obj is not None:
-            self.std_poll_timer = QTimer(self)
-            self.std_poll_timer.timeout.connect(self.poll_std_file_change)
-            self.std_poll_timer.setInterval(1000)
-            self.std_poll_timer.start()
             self.shellwidget.executed.connect(self.poll_std_file_change)
 
         self.start_successful = False
@@ -222,6 +224,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         # To show if special console is valid
         self._check_special_console_error()
 
+        # Set the initial current working directory in the kernel
+        self._set_initial_cwd_in_kernel()
+
         self.shellwidget.sig_prompt_ready.disconnect(
             self._when_prompt_is_ready)
         self.shellwidget.sig_remote_execute.disconnect(
@@ -254,16 +259,18 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def _show_loading_page(self):
         """Show animation while the kernel is loading."""
-        self.shellwidget.hide()
-        self.infowidget.show()
-        self.info_page = self.loading_page
-        self.set_info_page()
+        if self.infowidget is not None:
+            self.shellwidget.hide()
+            self.infowidget.show()
+            self.info_page = self.loading_page
+            self.set_info_page()
 
     def _hide_loading_page(self):
         """Hide animation shown while the kernel is loading."""
-        self.infowidget.hide()
-        self.info_page = self.blank_page
-        self.set_info_page()
+        if self.infowidget is not None:
+            self.infowidget.hide()
+            self.info_page = self.blank_page
+            self.set_info_page()
         self.shellwidget.show()
 
     def _read_stderr(self):
@@ -321,11 +328,16 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
         We also ignore errors about comms, which are irrelevant.
         """
+        if not self.has_spyder_kernels():
+            return True
+
         if self.start_successful:
             return False
+
         stderr = self.stderr_obj.get_contents()
         if not stderr:
             return False
+
         # There is an error. If it is benign, ignore.
         for line in stderr.splitlines():
             if line and not self.is_benign_error(line):
@@ -348,6 +360,58 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         page_control.sig_show_find_widget_requested.connect(
             self.container.find_widget.show)
 
+    def _set_initial_cwd_in_kernel(self):
+        """Set the initial cwd in the kernel."""
+        logger.debug("Setting initial working directory in the kernel")
+        cwd_path = get_home_dir()
+        project_path = self.container.get_active_project_path()
+        emit_cwd_change = True
+
+        # This is for the first client
+        if self.id_['int_id'] == '1':
+            if self.get_conf(
+                'startup/use_project_or_home_directory',
+                section='workingdir'
+            ):
+                cwd_path = get_home_dir()
+                if project_path is not None:
+                    cwd_path = project_path
+            elif self.get_conf(
+                'startup/use_fixed_directory',
+                section='workingdir'
+            ):
+                cwd_path = self.get_conf(
+                    'startup/fixed_directory',
+                    default=get_home_dir(),
+                    section='workingdir'
+                )
+        else:
+            # For new clients
+            if self.initial_cwd is not None:
+                cwd_path = self.initial_cwd
+            elif self.get_conf(
+                'console/use_project_or_home_directory',
+                section='workingdir'
+            ):
+                cwd_path = get_home_dir()
+                if project_path is not None:
+                    cwd_path = project_path
+            elif self.get_conf('console/use_cwd', section='workingdir'):
+                cwd_path = self.container.get_working_directory()
+                emit_cwd_change = False
+            elif self.get_conf(
+                'console/use_fixed_directory',
+                section='workingdir'
+            ):
+                cwd_path = self.get_conf(
+                    'console/fixed_directory',
+                    default=get_home_dir(),
+                    section='workingdir'
+                )
+
+        if osp.isdir(cwd_path):
+            self.shellwidget.set_cwd(cwd_path, emit_cwd_change=emit_cwd_change)
+
     # ----- Public API --------------------------------------------------------
     @property
     def kernel_id(self):
@@ -360,10 +424,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         """Remove stderr_file associated with the client."""
         try:
             self.shellwidget.executed.disconnect(self.poll_std_file_change)
-        except TypeError:
+        except (TypeError, RuntimeError):
             pass
-        if self.std_poll_timer is not None:
-            self.std_poll_timer.stop()
         if is_last_client:
             if self.stderr_obj is not None:
                 self.stderr_obj.remove()
@@ -375,7 +437,6 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
     @Slot()
     def poll_std_file_change(self):
         """Check if the stderr or stdout file just changed."""
-        self.shellwidget.call_kernel().flush_std()
         starting = self.shellwidget._starting
         if self.stderr_obj is not None:
             stderr = self.stderr_obj.poll_file_change()
@@ -507,9 +568,10 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             error=error)
 
         # Show error
-        self.set_info_page()
-        self.shellwidget.hide()
-        self.infowidget.show()
+        if self.infowidget is not None:
+            self.set_info_page()
+            self.shellwidget.hide()
+            self.infowidget.show()
 
         # Tell the client we're in error mode
         self.is_error_shown = True
@@ -535,7 +597,12 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             # disable secure writes.
             "WARNING: Insecure writes have been enabled via environment",
             # Old error
-            "No such comm"
+            "No such comm",
+            # PYDEVD debug warning message. See spyder-ide/spyder#18908
+            "Note: Debugging will proceed. "
+            "Set PYDEVD_DISABLE_FILE_VALIDATION=1 to disable this validation.",
+            # Argument not expected error. See spyder-ide/spyder#19298
+            "The following argument was not expected"
         ]
 
         return any([err in error for err in benign_errors])
@@ -591,6 +658,25 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         except AttributeError:
             pass
 
+    def close_client(self, is_last_client):
+        """Close the client."""
+        # Needed to handle a RuntimeError. See spyder-ide/spyder#5568.
+        try:
+            # Close client
+            self.stop_button_click_handler()
+        except RuntimeError:
+            pass
+
+        # Disconnect timer needed to update elapsed time
+        try:
+            self.timer.timeout.disconnect(self.show_time)
+        except (RuntimeError, TypeError):
+            pass
+
+        self.shutdown(is_last_client)
+        self.close()
+        self.setParent(None)
+
     def shutdown(self, is_last_client):
         """Shutdown connection and kernel if needed."""
         self.dialog_manager.close_all()
@@ -636,8 +722,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                 running_under_pytest() or
                 not self.ask_before_restart):
             if sw.kernel_manager:
-                if self.infowidget.isVisible():
-                    self.infowidget.hide()
+                if self.infowidget is not None:
+                    if self.infowidget.isVisible():
+                        self.infowidget.hide()
 
                 if self._abort_kernel_restart():
                     sw.spyder_kernel_comm.close()
@@ -734,6 +821,56 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.restart_thread = None
         self.sig_execution_state_changed.emit()
 
+    def has_spyder_kernels(self):
+        """
+        Check if the kernel has the right Spyder-kernels version and show a
+        message in case that's not the true.
+        """
+        if not self.get_conf('default', section='main_interpreter'):
+            pyexec = self.get_conf('executable', section='main_interpreter')
+            has_spyder_kernels = programs.is_module_installed(
+                'spyder_kernels',
+                interpreter=pyexec,
+                version=SPYDER_KERNELS_VERSION)
+
+            if (
+                not has_spyder_kernels
+                # This is necessary to show this message for some tests and not
+                # do it for others.
+                and os.environ.get('SPY_TEST_SHOW_RESTART_MESSAGE')
+            ):
+                self.show_kernel_error(
+                    _("The Python environment or installation whose "
+                      "interpreter is located at"
+                      "<pre>"
+                      "    <tt>{0}</tt>"
+                      "</pre>"
+                      "doesn't have the <tt>spyder-kernels</tt> module or the "
+                      "right version of it installed ({1}). "
+                      "Without this module is not possible for Spyder to "
+                      "create a console for you.<br><br>"
+                      "You can install it by activating your environment (if "
+                      "necessary) and then running in a system terminal:"
+                      "<pre>"
+                      "    <tt>{2}</tt>"
+                      "</pre>"
+                      "or"
+                      "<pre>"
+                      "    <tt>{3}</tt>"
+                      "</pre>").format(
+                          pyexec,
+                          SPYDER_KERNELS_VERSION_MSG,
+                          SPYDER_KERNELS_CONDA,
+                          SPYDER_KERNELS_PIP
+                      )
+                )
+
+                return False
+            else:
+                return True
+        else:
+            return True
+
     def filter_fault(self, fault):
         """Get a fault from a previous session."""
         thread_regex = (
@@ -845,8 +982,6 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def show_time(self, end=False):
         """Text to show in time_label."""
-        if self.time_label is None:
-            return
 
         elapsed_time = time.monotonic() - self.t0
         # System time changed to past date, so reset start.
@@ -865,9 +1000,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                "</b></span>" % (color,
                                 time.strftime(fmt, time.gmtime(elapsed_time)))
         if self.show_elapsed_time:
-            self.time_label.setText(text)
+            self.sig_time_label.emit(text)
         else:
-            self.time_label.setText("")
+            self.sig_time_label.emit("")
 
     @Slot(bool)
     def set_show_elapsed_time(self, state):
@@ -876,7 +1011,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def set_info_page(self):
         """Set current info_page."""
-        if self.info_page is not None:
+        if self.infowidget is not None and self.info_page is not None:
             self.infowidget.setHtml(
                 self.info_page,
                 QUrl.fromLocalFile(self.css_path)
